@@ -10,6 +10,7 @@
 #include <libHX/list.h>
 #include <libHX/init.h>
 #include <libHX/misc.h>
+#include "internal.h"
 
 struct text_object {
 	struct HXlist_head list;
@@ -22,7 +23,7 @@ union list_encap {
 
 static HXCLIST_HEAD(strings_ct);
 
-static void l_init(unsigned int max, bool unshift)
+static int l_init(unsigned int max, bool unshift)
 {
 	static const char *const msg[] = {"Pushing", "Unshifting"};
 	struct text_object *obj;
@@ -34,7 +35,7 @@ static void l_init(unsigned int max, bool unshift)
 #else
 		obj = malloc(sizeof(*obj));
 		if (obj == NULL)
-			abort();
+			return EXIT_FAILURE;
 #endif
 		HXlist_init(&obj->list);
 		obj->id[0] = HX_irand('a', 'z'+1);
@@ -48,6 +49,7 @@ static void l_init(unsigned int max, bool unshift)
 		else
 			HXclist_push(&strings_ct, &obj->list);
 	}
+	return EXIT_SUCCESS;
 }
 
 static void l_traverse(void)
@@ -63,13 +65,14 @@ static void l_traverse(void)
 static void l_dump(bool pop)
 {
 	static const char *const msg[] = {"Shifting", "Popping"};
-	struct text_object *obj;
 	unsigned int i = 0;
 
-	while ((obj = (pop ?
-	    HXclist_pop(&strings_ct, struct text_object, list) :
-	    HXclist_shift(&strings_ct, struct text_object, list)
-	    )) != NULL) {
+	while (true) {
+		struct text_object *obj = pop ?
+			HXclist_pop(&strings_ct, struct text_object, list) :
+			HXclist_shift(&strings_ct, struct text_object, list);
+		if (obj == nullptr)
+			break;
 		printf("%s item %u (\"%s\")\n", msg[pop], ++i, obj->id);
 #ifdef __cplusplus
 		delete obj;
@@ -137,16 +140,17 @@ static void l_shift(void)
 #pragma GCC diagnostic pop
 }
 
-int main(int argc, const char **argv)
+static int runner(int argc, char **argv)
 {
 	unsigned int max = 10;
 
 	if (HX_init() <= 0)
-		abort();
+		return EXIT_FAILURE;
 	if (argc >= 2)
 		max = strtoul(argv[1], NULL, 0);
-
-	l_init(max, HX_rand() & 1);
+	int ret = l_init(max, HX_rand() & 1);
+	if (ret != EXIT_SUCCESS)
+		return ret;
 	l_traverse();
 	l_dump(HX_rand() & 1);
 	l_empty();
@@ -154,3 +158,11 @@ int main(int argc, const char **argv)
 	HX_exit();
 	return EXIT_SUCCESS;
 }
+
+int main(int argc, char **argv)
+{
+	int ret = runner(argc, argv);
+	if (ret == EXIT_FAILURE)
+		fprintf(stderr, "FAILED\n");
+	return ret;
+}
